 /*****************************************************************************/
/** \mainpage Cal3D API Reference
  * <center>
  *   <p>
  *     Permission is granted to copy, distribute and/or modify this document
  *     under the terms of the GNU Free Documentation License, Version 1.1 or
  *     any later version published by the Free Software Foundation;
  *     with no Invariant Sections, no Front-Cover Texts and
  *     no Back-Cover Texts;
  *     A copy of the license is included in the section entitled
  *     \link license "GNU Free Documentation License" \endlink .
  * </center>
  *****************************************************************************/
