/* false positive from findutils */
  static const char *copy_para[]=
    {
      "Copyright (C) 1994, 1996, 1998, 2000, 2001, 2003, 2004, 2005, 2006,"
      ,"2007, 2009, 2010, 2011 Free Software Foundation, Inc."
      ,""
      ,"Permission is granted to copy, distribute and/or modify this document"
      ,"under the terms of the GNU Free Documentation License, Version 1.3 or"
      ,"any later version published by the Free Software Foundation; with no"
      ,"Invariant Sections, with no Front-Cover Texts, and with no Back-Cover"
      ,"Texts.  A copy of the license is included in the ``GNU Free"
      ,"Documentation License'' file as part of this distribution."
      ""
      ,NULL
    };
